#!/bin/bash
#
# VERSION=4
# CHANGES="get trap-community from /usr/conf/snmpd_berofix.conf"

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:/usr/fallback

OID_BASE=.1.3.6.1.4.1.29886.2.0

SNMPTRAP=/usr/local/bin/snmptrap
QUERYISGW=/usr/local/sbin/query_isgw

if [ -z "${1}" ] || [ -z "${2}" ]; then
	echo "Usage: ${0} <wait> <sink ipaddr>"
	exit 1
fi

if [ "${1}" = "0" ]; then
	let "wait_time=60"
else
	let "wait_time=60 * ${1}"
fi

sink_addr=${2}

trap_community=$(expr match "$(grep trapcommunity /usr/conf/snmpd_berofix.conf)" ".*trapcommunity[\ ]*\([0-9a-zA-Z]*\)")
if [ -z "${trap_community}" ]; then
	trap_community=public
fi

# main loop
while [ 1 ]; do

	sleep ${wait_time}

	# gather ports configured
	max_port=0
	port_list=$(${QUERYISGW} i | grep "* Port")

	for i in $(seq 1 32); do
		res=$(expr match "${port_list}" ".* Port \(${i}\)")
		if [ -z "${res}" ]; then
			break
		fi

		let "max_port=${max_port}+1"
	done

	if [ -z "${max_port}" ] || [ ${max_port} -eq 0 ]; then
		continue
	fi


	# try to bring ports up
	i=1
	while [ ${i} -le ${max_port} ]; do
		${QUERYISGW} l2up${i} > /dev/null
		let "i=${i}+1"
	done


	# wait a second
	sleep 1


	# check port-status & send notifications
	i=1
	while [ ${i} -le ${max_port} ]; do
		res_str=$(${QUERYISGW} i | grep "* Port ${i}")
		l2s_str=$(expr match "${res_str}" ".*L2Link \([a-zA-Z]*\)")

		case "${l2s_str}" in
			UP)
				# State of L2 is UP
				if [ "${prv_state[${i}]}" != "UP" ]; then
					${SNMPTRAP} -v 2c -c ${trap_community} ${sink_addr} '' ${OID_BASE} ${OID_BASE}.0 i ${i} ${OID_BASE}.1 i 1 ${OID_BASE}.2 i 0
				fi
				;;
			DOWN)
				# State of L2 is DOWN
				${SNMPTRAP} -v 2c -c ${trap_community} ${sink_addr} '' ${OID_BASE} ${OID_BASE}.0 i ${i} ${OID_BASE}.1 i 0 ${OID_BASE}.2 i 1
				;;
			*)
				# State of L2 is unknown
				${SNMPTRAP} -v 2c -c ${trap_community} ${sink_addr} '' ${OID_BASE} ${OID_BASE}.0 i ${i} ${OID_BASE}.1 i 0 ${OID_BASE}.2 i 0
				;;
		esac

		prv_state[${i}]=${l2s_str}

		let "i=${i}+1"
	done

done
